@@ -4,12 +4,11 @@ use 5.010;
 use strict;
 use warnings;
 
-use base 'Exporter';
-our @EXPORT_OK = qw/pkg/;
-
+use base 'Csistck::Test';
 use Csistck::Oper qw/debug/;
 use Csistck::Config qw/option/;
-use Csistck::Test;
+
+our @EXPORT_OK = qw/pkg/;
 
 use Digest::MD5;
 use File::Basename;
@@ -51,7 +50,7 @@ Csistck::Test::Pkg - Csistck package check
 
 =head1 METHODS
 
-=head2 pkg($package, [$type])
+=head2 pkg($package, $type, :\&on_repair)
 
 Test for existing package using forks to system package managers. Package can be
 specified as a string, or as a hashref:
@@ -69,6 +68,8 @@ name to check. If a default key is provided, that package is used by default.
 In repair mode, install the package quietly, unless package manager doesn't
 handle automating install.
 
+If a repair operation is run, the on_repair function is called
+
 Supported package managers:
 
 =over
@@ -88,75 +89,98 @@ Arch linux package management utility
 =cut 
 
 sub pkg {
-    my $pkgref = shift;
-    my $type = shift;
-    my $pkg;
-    my $diff;
+    my $pkg = shift;
+    my $type = shift // undef;
+    my %args = (
+        type => $type,
+        @_
+    );
+    Csistck::Test::Pkg->new($pkg, %args);
+}
 
+sub new {
+    my $class = shift;
+    my $self = $class->SUPER::new(@_);
+    bless($self, $class);
+
+    # We'll fix package name here, instead of redoing each call
     # Priority: 'type' argument, 'pkg_type' option, detect_pkg_manager.
+    my $type = $self->{type};
     if (! $type) {
         $type = option('pkg_type') // detect_pkg_manager();
     }
-    die("Bad package manager: manager=<$type>")
+    return $self->fail("Unsupported package manager or OS: manager=<none>")
       if (! $type);
-    die("Package manager not supported: type=<$type>")
+    return $self->fail("Package manager not supported: type=<$type>")
       if (! $Cmds->{$type});
-    
-    $pkg = get_pkg($pkgref, $type);
-    $diff = sub { pkg_diff($pkg, $type); }
-      if ($Cmds->{$type}->{diff});
-    
-    return Csistck::Test->new(
-        check => sub { pkg_check($pkg, $type); },
-        repair => sub { pkg_install($pkg, $type); },
-        diff => $diff,
-        desc => "Searching for package $pkg, using $type"
-    );
+    $self->{type} = $type;
+
+    return $self;
 }
 
-sub pkg_check {
-    my ($pkg, $type) = @_;
+sub desc {
+    return sprintf("Package test for %s, using %s", 
+      $_[0]->pkg_name, $_[0]->pkg_type);
+}
+
+sub check {
+    my $self = shift;
+    my $pkg = $self->pkg_name;
+    my $type = $self->pkg_type;
     my $cmd = sprintf($Cmds->{$type}->{check}, $pkg) or
-      die("Package check command missing: type=<$type>");
+      return $self->fail("Package check command missing: type=<$type>");
 
     debug("Searching for package via command: cmd=<$cmd>");    
     my $ret = system("$cmd 1>/dev/null 2>/dev/null");
 
-    die("Package missing")
+    return $self->fail("Package missing")
       unless($ret == 0);
+
+    return $self->pass("Package installed");
 }
 
-# Package install
-sub pkg_install {
-    my ($pkg, $type) = @_;
+sub repair {
+    my $self = shift;
+    my $pkg = $self->pkg_name;
+    my $type = $self->pkg_type;
     my $cmd;
 
     if (defined $Cmds->{$type}->{install}) {
         $cmd = sprintf($Cmds->{$type}->{install}, $pkg);
     }
     else {
-        die("Package install command missing: type=<$type>");
+        return $self->fail("Package install command missing: type=<$type>");
     }
 
     $ENV{DEBIAN_FRONTEND} = "noninteractive";
     debug("Installing package via command: cmd=<$cmd>");
     my $ret = system("$cmd 1>/dev/null 2>/dev/null");
 
-    die("Package installation failed")
+    return $self->fail("Package installation failed")
       unless ($ret == 0);
+    
+    return $self->pass("Package installation successful");
 }
 
 # Package diff
-sub pkg_diff {
-    my ($pkg, $type) = @_;
-    my $cmd = sprintf($Cmds->{$type}->{diff}, $pkg) or
-      die("Package diff command missing: type=<$type>");
+sub diff {
+    my $self = shift;
+    my $pkg = $self->pkg_name;
+    my $type = $self->pkg_type;
+    my $cmd;
+    
+    if (defined $Cmds->{$type}->{diff}) {
+        $cmd = sprintf($Cmds->{$type}->{diff}, $pkg);
+    }
+    else {
+        return $self->fail("Package diff command missing: type=<$type>");
+    }
     
     $ENV{DEBIAN_FRONTEND} = "noninteractive";
     debug("Showing package differences via command: cmd=<$cmd>");
     my $ret = system("$cmd 2>/dev/null");
 
-    die("Package differences query failed")
+    return $self->fail("Package differences query failed")
       unless ($ret == 0);
 }
 
@@ -169,6 +193,7 @@ used for the package test.
 =cut
 
 sub detect_pkg_manager {
+    my $self = shift;
     given ("$^O") {
         when (/^freebsd$/) { return 'pkg_info'; }
         when (/^netbsd$/) { return 'pkg_info'; }
@@ -186,7 +211,7 @@ sub detect_pkg_manager {
     }
 }
 
-=head2 get_pkg($package, $type)
+=head2 pkg_name($package, $type)
 
 Based on input package, return package name. With OS and distribution detection,
 $package can be passed as a string or a hashref.
@@ -195,13 +220,11 @@ See pkg() for information on passing hashrefs as package name
 
 =cut
 
-sub get_pkg {
-    my $pkg = shift;
-    my $type = shift;
-    
-    debug(sprintf('Searching for package: ref=<%s>, pkg=<%s>', 
-      ref $pkg, $pkg));
-    
+sub pkg_name {
+    my $self = shift;
+    my $pkg = $self->{target};
+    my $type = $self->{type};
+
     given (ref $pkg) {
         when ('') {
             return $pkg if ($pkg =~ m/^[A-Za-z0-9\-\_\.\/]+$/);
@@ -215,6 +238,14 @@ sub get_pkg {
     die('Invalid package');
 }
 
+=head2 pkg_type()
+
+Return fixed package type
+
+=cut
+
+sub pkg_type { $_[0]->{type}; }
+
 1;
 __END__
 
